@@ -26,7 +26,6 @@
 #include <string.h>
 #include <time.h>
 
-#define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus-glib.h>
 
 #include <telepathy-glib/dbus.h>
@@ -36,6 +35,8 @@
 #include <telepathy-glib/simple-password-manager.h>
 #include <telepathy-glib/svc-connection.h>
 #include <telepathy-glib/channel-manager.h>
+#include <telepathy-glib/gtypes.h>
+#include <telepathy-glib/util.h>
 
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_CONNECTION
 #include "idle-contact-info.h"
@@ -46,8 +47,6 @@
 #include "idle-muc-manager.h"
 #include "idle-parser.h"
 #include "idle-server-connection.h"
-#include "idle-server-connection-iface.h"
-#include "idle-ssl-server-connection.h"
 
 #include "extensions/extensions.h"    /* Renaming */
 
@@ -61,18 +60,15 @@
 
 #define MSG_QUEUE_UNLOAD_AT_A_TIME 1
 #define MSG_QUEUE_TIMEOUT 2
+static gboolean flush_queue_faster = FALSE;
 
 #define SERVER_CMD_MIN_PRIORITY 0
 #define SERVER_CMD_NORMAL_PRIORITY G_MAXUINT/2
 #define SERVER_CMD_MAX_PRIORITY G_MAXUINT
 
-/* FIXME use this from telepathy-glib as soon as it gets there */
-#define IDLE_TP_ALIAS_PAIR_TYPE (dbus_g_type_get_struct ("GValueArray", \
-			G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID))
-
 static void _free_alias_pair(gpointer data, gpointer user_data)
 {
-	g_boxed_free(IDLE_TP_ALIAS_PAIR_TYPE, data);
+	g_boxed_free(TP_STRUCT_TYPE_ALIAS_PAIR, data);
 }
 
 static void _aliasing_iface_init(gpointer, gpointer);
@@ -81,15 +77,31 @@ static void _renaming_iface_init(gpointer, gpointer);
 G_DEFINE_TYPE_WITH_CODE(IdleConnection, idle_connection, TP_TYPE_BASE_CONNECTION,
 		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CONNECTION_INTERFACE_ALIASING, _aliasing_iface_init);
 		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CONNECTION_INTERFACE_CONTACT_INFO, idle_contact_info_iface_init);
-		G_IMPLEMENT_INTERFACE(IDLE_TYPE_SVC_CONNECTION_INTERFACE_RENAMING, _renaming_iface_init));
+		G_IMPLEMENT_INTERFACE(IDLE_TYPE_SVC_CONNECTION_INTERFACE_RENAMING, _renaming_iface_init);
+		G_IMPLEMENT_INTERFACE(TP_TYPE_SVC_CONNECTION_INTERFACE_CONTACTS, tp_contacts_mixin_iface_init);
+);
 
 typedef struct _IdleOutputPendingMsg IdleOutputPendingMsg;
+typedef struct _MsgQueueTimeoutData MsgQueueTimeoutData;
+typedef struct _SendWithPriorityData SendWithPriorityData;
+
 struct _IdleOutputPendingMsg {
 	gchar *message;
 	guint priority;
 	guint64 id;
 };
 
+struct _MsgQueueTimeoutData {
+	IdleConnection *conn;
+	GSList *sent_msgs;
+};
+
+struct _SendWithPriorityData {
+	IdleConnection *conn;
+	gchar *msg;
+	guint priority;
+};
+
 static IdleOutputPendingMsg *idle_output_pending_msg_new() {
 	IdleOutputPendingMsg *msg = g_slice_new(IdleOutputPendingMsg);
 	static guint64 last_id = 0;
@@ -141,7 +153,7 @@ struct _IdleConnectionPrivate {
 	 * network connection
 	 */
 
-	IdleServerConnectionIface *conn;
+	IdleServerConnection *conn;
 	guint sconn_status;
 
 	/* IRC connection properties */
@@ -166,6 +178,9 @@ struct _IdleConnectionPrivate {
 	/* output message queue */
 	GQueue *msg_queue;
 
+	/* has it submitted a message for sending and waiting for acknowledgement */
+	gboolean msg_sending;
+
 	/* UNIX time the last message was sent on */
 	time_t last_msg_sent;
 
@@ -208,8 +223,8 @@ static IdleParserHandlerResult _version_privmsg_handler(IdleParser *parser, Idle
 static IdleParserHandlerResult _welcome_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 static IdleParserHandlerResult _whois_user_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 
-static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServerConnectionState state, IdleServerConnectionStateReason reason, IdleConnection *conn);
-static void sconn_received_cb(IdleServerConnectionIface *sconn, gchar *raw_msg, IdleConnection *conn);
+static void sconn_status_changed_cb(IdleServerConnection *sconn, IdleServerConnectionState state, IdleServerConnectionStateReason reason, IdleConnection *conn);
+static void sconn_received_cb(IdleServerConnection *sconn, gchar *raw_msg, IdleConnection *conn);
 
 static void irc_handshakes(IdleConnection *conn);
 static void send_quit_request(IdleConnection *conn);
@@ -218,23 +233,35 @@ static void connection_disconnect_cb(IdleConnection *conn, TpConnectionStatusRea
 static gboolean idle_connection_hton(IdleConnection *obj, const gchar *input, gchar **output, GError **_error);
 static void idle_connection_ntoh(IdleConnection *obj, const gchar *input, gchar **output);
 
+static void idle_connection_add_queue_timeout (IdleConnection *self);
+static void idle_connection_clear_queue_timeout (IdleConnection *self);
+
 static void _send_with_priority(IdleConnection *conn, const gchar *msg, guint priority);
+static void conn_aliasing_fill_contact_attributes (
+    GObject *obj,
+    const GArray *contacts,
+    GHashTable *attributes_hash);
 
 static void idle_connection_init(IdleConnection *obj) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(obj);
 
 	priv->sconn_status = SERVER_CONNECTION_STATE_NOT_CONNECTED;
 	priv->msg_queue = g_queue_new();
+
+	tp_contacts_mixin_init ((GObject *) obj, G_STRUCT_OFFSET (IdleConnection, contacts));
+	tp_base_connection_register_with_contacts_mixin ((TpBaseConnection *) obj);
 }
 
-static GObject *idle_connection_constructor(GType type, guint n_params, GObjectConstructParam *params) {
-	IdleConnection *self = IDLE_CONNECTION(G_OBJECT_CLASS(idle_connection_parent_class)->constructor(type, n_params, params));
+static void
+idle_connection_constructed (GObject *object)
+{
+  IdleConnection *self = IDLE_CONNECTION (object);
 
-	self->parser = g_object_new(IDLE_TYPE_PARSER, "connection", self, NULL);
-
-	idle_contact_info_init(self);
-
-	return (GObject *) self;
+  self->parser = g_object_new (IDLE_TYPE_PARSER, "connection", self, NULL);
+  idle_contact_info_init (self);
+  tp_contacts_mixin_add_contact_attributes_iface (object,
+      TP_IFACE_CONNECTION_INTERFACE_ALIASING,
+      conn_aliasing_fill_contact_attributes);
 }
 
 static void idle_connection_set_property(GObject *obj, guint prop_id, const GValue *value, GParamSpec *pspec) {
@@ -405,6 +432,7 @@ static void idle_connection_finalize (GObject *object) {
 		idle_output_pending_msg_free(msg);
 
 	g_queue_free(priv->msg_queue);
+	tp_contacts_mixin_finalize (object);
 
 	G_OBJECT_CLASS(idle_connection_parent_class)->finalize(object);
 }
@@ -414,6 +442,7 @@ static const gchar * interfaces_always_present[] = {
 	TP_IFACE_CONNECTION_INTERFACE_CONTACT_INFO,
 	IDLE_IFACE_CONNECTION_INTERFACE_RENAMING,
 	TP_IFACE_CONNECTION_INTERFACE_REQUESTS,
+	TP_IFACE_CONNECTION_INTERFACE_CONTACTS,
 	NULL};
 
 const gchar * const *idle_connection_get_implemented_interfaces (void) {
@@ -428,7 +457,7 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 
 	g_type_class_add_private(klass, sizeof(IdleConnectionPrivate));
 
-	object_class->constructor = idle_connection_constructor;
+	object_class->constructed = idle_connection_constructed;
 	object_class->set_property = idle_connection_set_property;
 	object_class->get_property = idle_connection_get_property;
 	object_class->dispose = idle_connection_dispose;
@@ -478,7 +507,12 @@ static void idle_connection_class_init(IdleConnectionClass *klass) {
 	param_spec = g_param_spec_boolean("password-prompt", "Password prompt", "Whether the connection should pop up a SASL channel if no password is given", FALSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_CONSTRUCT);
 	g_object_class_install_property(object_class, PROP_PASSWORD_PROMPT, param_spec);
 
+	tp_contacts_mixin_class_init (object_class, G_STRUCT_OFFSET (IdleConnectionClass, contacts));
 	idle_contact_info_class_init(klass);
+
+	/* This is a hack to make the test suite run in finite time. */
+	if (!tp_str_empty (g_getenv ("IDLE_HTFU")))
+		flush_queue_faster = TRUE;
 }
 
 static GPtrArray *_iface_create_channel_managers(TpBaseConnection *self) {
@@ -528,7 +562,7 @@ _force_disconnect (gpointer data)
 {
 	IdleConnection *conn = IDLE_CONNECTION(data);
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
-	idle_server_connection_iface_disconnect(priv->conn, NULL);
+	idle_server_connection_disconnect_async(priv->conn, NULL, NULL, NULL);
 	return FALSE;
 }
 
@@ -537,7 +571,8 @@ static void _iface_disconnected(TpBaseConnection *self) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
 
 	/* we never got around to actually creating the connection
-         * iface object, so don't try to send any traffic down it */
+	 * iface object because we were still trying to connect, so
+	 * don't try to send any traffic down it */
 	if (priv->conn == NULL) {
 		return;
 	}
@@ -554,19 +589,38 @@ static void _iface_disconnected(TpBaseConnection *self) {
 
 static void _iface_shut_down(TpBaseConnection *self) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(self);
-	GError *error = NULL;
 
 	if (priv->quitting)
 		return;
 
-	if (priv->sconn_status != SERVER_CONNECTION_STATE_NOT_CONNECTED) {
-		if (!idle_server_connection_iface_disconnect(priv->conn, &error))
-			g_error_free(error);
-	} else {
+	/* we never got around to actually creating the connection
+	 * iface object because we were still trying to connect, so
+	 * don't try to send any traffic down it */
+	if (priv->conn == NULL) {
 		g_idle_add(_finish_shutdown_idle_func, self);;
+	} else {
+		idle_server_connection_disconnect_async(priv->conn, NULL, NULL, NULL);
 	}
 }
 
+static void _connection_disconnect_with_gerror(IdleConnection *conn, TpConnectionStatusReason reason, const gchar *key, const GError *error) {
+	GHashTable *details = tp_asv_new(key, G_TYPE_STRING, error->message, NULL);
+
+	g_assert(error->domain == TP_ERRORS);
+	tp_base_connection_disconnect_with_dbus_error(TP_BASE_CONNECTION(conn),
+						      tp_error_get_dbus_name(error->code),
+						      details,
+						      reason);
+	g_hash_table_unref(details);
+}
+
+static void _connection_disconnect_with_error(IdleConnection *conn, TpConnectionStatusReason reason, const gchar *key, GQuark domain, gint code, const gchar *message) {
+	GError *error = g_error_new_literal(domain, code, message);
+
+	_connection_disconnect_with_gerror(conn, reason, key, error);
+	g_error_free(error);
+}
+
 static void _start_connecting_continue(IdleConnection *conn);
 
 static void _password_prompt_cb(GObject *source, GAsyncResult *result, gpointer user_data) {
@@ -581,12 +635,11 @@ static void _password_prompt_cb(GObject *source, GAsyncResult *result, gpointer
 	if (error != NULL) {
 		IDLE_DEBUG("Simple password manager failed: %s", error->message);
 
-		if (base_conn->status != TP_CONNECTION_STATUS_DISCONNECTED) {
-			tp_base_connection_disconnect_with_dbus_error(base_conn,
-								      tp_error_get_dbus_name(error->code),
-								      NULL,
-								      TP_CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED);
-		}
+		if (base_conn->status != TP_CONNECTION_STATUS_DISCONNECTED)
+			_connection_disconnect_with_gerror(conn,
+							   TP_CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED,
+							   "debug-message",
+							   error);
 
 		g_error_free(error);
 		return;
@@ -621,41 +674,16 @@ static gboolean _iface_start_connecting(TpBaseConnection *self, GError **error)
 	return TRUE;
 }
 
-static void _start_connecting_continue(IdleConnection *conn) {
-	TpBaseConnection *base_conn = TP_BASE_CONNECTION(conn);
+static void _connection_connect_ready(GObject *source_object, GAsyncResult *res, gpointer user_data) {
+	IdleServerConnection *sconn = IDLE_SERVER_CONNECTION(source_object);
+	IdleConnection *conn = IDLE_CONNECTION(user_data);
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
-	GError *conn_error = NULL;
-	IdleServerConnectionIface *sconn;
-	GType connection_type = (priv->use_ssl) ? IDLE_TYPE_SSL_SERVER_CONNECTION : IDLE_TYPE_SERVER_CONNECTION;
-
-	if (!priv->realname || !priv->realname[0]) {
-		const gchar *g_realname = g_get_real_name();
-
-		g_free(priv->realname);
-
-		if (g_realname && g_realname[0] && strcmp(g_realname, "Unknown"))
-			priv->realname = g_strdup(g_realname);
-		else
-			priv->realname = g_strdup(priv->nickname);
-	}
-
-	if (!priv->username || !priv->username[0]) {
-		g_free(priv->username);
-		priv->username = g_strdup(g_get_user_name());
-	}
-
-	sconn = IDLE_SERVER_CONNECTION_IFACE(g_object_new(connection_type, "host", priv->server, "port", priv->port, NULL));
-
-	g_signal_connect(sconn, "status-changed", (GCallback)(sconn_status_changed_cb), conn);
-
-	if (!idle_server_connection_iface_connect(sconn, &conn_error)) {
-		IDLE_DEBUG("server connection failed to connect: %s", conn_error->message);
-
-		tp_base_connection_disconnect_with_dbus_error(base_conn,
-							      TP_ERROR_STR_NETWORK_ERROR,
-							      NULL,
-							      TP_CONNECTION_STATUS_REASON_NETWORK_ERROR);
+	GError *error = NULL;
 
+	if (!idle_server_connection_connect_finish(sconn, res, &error)) {
+		IDLE_DEBUG("idle_server_connection_connect failed: %s", error->message);
+		_connection_disconnect_with_gerror(conn, TP_CONNECTION_STATUS_REASON_NETWORK_ERROR, "debug-message", error);
+		g_error_free(error);
 		g_object_unref(sconn);
 		return;
 	}
@@ -678,10 +706,38 @@ static void _start_connecting_continue(IdleConnection *conn) {
 	irc_handshakes(conn);
 }
 
-static gboolean keepalive_timeout_cb(gpointer user_data);
-static gboolean msg_queue_timeout_cb(gpointer user_data);
+static void _start_connecting_continue(IdleConnection *conn) {
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	IdleServerConnection *sconn;
 
-static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServerConnectionState state, IdleServerConnectionStateReason reason, IdleConnection *conn) {
+	if (tp_str_empty(priv->realname)) {
+		const gchar *g_realname = g_get_real_name();
+
+		g_free(priv->realname);
+
+		if (tp_strdiff(g_realname, "Unknown"))
+			priv->realname = g_strdup(g_realname);
+		else
+			priv->realname = g_strdup(priv->nickname);
+	}
+
+	if (tp_str_empty(priv->username)) {
+		g_free(priv->username);
+		priv->username = g_strdup(g_get_user_name());
+	}
+
+	sconn = g_object_new(IDLE_TYPE_SERVER_CONNECTION, "host", priv->server, "port", priv->port, NULL);
+	if (priv->use_ssl)
+		idle_server_connection_set_tls(sconn, TRUE);
+
+	g_signal_connect(sconn, "status-changed", (GCallback)(sconn_status_changed_cb), conn);
+
+	idle_server_connection_connect_async(sconn, NULL, _connection_connect_ready, conn);
+}
+
+static gboolean keepalive_timeout_cb(gpointer user_data);
+
+static void sconn_status_changed_cb(IdleServerConnection *sconn, IdleServerConnectionState state, IdleServerConnectionStateReason reason, IdleConnection *conn) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
 	TpConnectionStatusReason tp_reason;
 
@@ -713,13 +769,13 @@ static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServer
 
 	switch (state) {
 		case SERVER_CONNECTION_STATE_NOT_CONNECTED:
-			if (conn->parent.status == TP_CONNECTION_STATUS_CONNECTING) {
-				connection_connect_cb(conn, FALSE, TP_CONNECTION_STATUS_REASON_NETWORK_ERROR);
+			/* when the Disconnect method is called, the parent's status is
+			 * immediately set to disconnected, and when the underlying network
+			 * connection finally disconnects and triggers this callback
+			 * we need to finish shutting down the connection -- so we can only
+			 * ignore the case where the parent's status is connecting */
+			if (conn->parent.status != TP_CONNECTION_STATUS_CONNECTING)
 				connection_disconnect_cb(conn, tp_reason);
-			}
-			else {
-				connection_disconnect_cb(conn, tp_reason);
-			}
 			break;
 
 		case SERVER_CONNECTION_STATE_CONNECTING:
@@ -729,10 +785,9 @@ static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServer
 			if (priv->keepalive_interval != 0 && priv->keepalive_timeout == 0)
 				priv->keepalive_timeout = g_timeout_add_seconds(priv->keepalive_interval, keepalive_timeout_cb, conn);
 
-			if ((priv->msg_queue_timeout == 0) && (g_queue_get_length(priv->msg_queue) > 0)) {
+			if (g_queue_get_length(priv->msg_queue) > 0) {
 				IDLE_DEBUG("we had messages in queue, start unloading them now");
-
-				priv->msg_queue_timeout = g_timeout_add(MSG_QUEUE_TIMEOUT, msg_queue_timeout_cb, conn);
+				idle_connection_add_queue_timeout (conn);
 			}
 			break;
 
@@ -744,7 +799,7 @@ static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServer
 	priv->sconn_status = state;
 }
 
-static void sconn_received_cb(IdleServerConnectionIface *sconn, gchar *raw_msg, IdleConnection *conn) {
+static void sconn_received_cb(IdleServerConnection *sconn, gchar *raw_msg, IdleConnection *conn) {
 	gchar *converted;
 
 	idle_connection_ntoh(conn, raw_msg, &converted);
@@ -771,13 +826,44 @@ static gboolean keepalive_timeout_cb(gpointer user_data) {
 	return TRUE;
 }
 
+static void _sent_msgs_foreach_free(gpointer data) {
+	idle_output_pending_msg_free((IdleOutputPendingMsg *) data);
+}
+
+static void _restore_msg_to_queue(gpointer data, gpointer user_data) {
+	GQueue *msg_queue = (GQueue *) user_data;
+	g_queue_push_head(msg_queue, data);
+}
+
+static void _msg_queue_timeout_ready(GObject *source_object, GAsyncResult *res, gpointer user_data) {
+	IdleServerConnection *sconn = IDLE_SERVER_CONNECTION(source_object);
+	MsgQueueTimeoutData *data = (MsgQueueTimeoutData *) user_data;
+	IdleConnection *conn = data->conn;
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	GSList *sent_msgs = data->sent_msgs;
+	GError *error = NULL;
+
+	g_slice_free(MsgQueueTimeoutData, data);
+	priv->msg_sending = FALSE;
+
+	if (!idle_server_connection_send_finish(sconn, res, &error)) {
+		IDLE_DEBUG("idle_server_connection_send failed: %s", error->message);
+		g_error_free(error);
+		g_slist_foreach(sent_msgs, _restore_msg_to_queue, priv->msg_queue);
+		return;
+	}
+
+	g_slist_free_full(sent_msgs, _sent_msgs_foreach_free);
+	priv->last_msg_sent = time(NULL);
+}
+
 static gboolean msg_queue_timeout_cb(gpointer user_data) {
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
-	int i, j;
+	int i;
 	IdleOutputPendingMsg *output_msg;
+	MsgQueueTimeoutData *data;
 	gchar msg[IRC_MSG_MAXLEN + 3];
-	GError *error = NULL;
 
 	IDLE_DEBUG("called");
 
@@ -789,17 +875,25 @@ static gboolean msg_queue_timeout_cb(gpointer user_data) {
 		return FALSE;
 	}
 
-	output_msg = g_queue_peek_head(priv->msg_queue);
+	if (priv->msg_sending)
+		return TRUE;
+
+	output_msg = g_queue_pop_head(priv->msg_queue);
 
 	if (output_msg == NULL) {
 		priv->msg_queue_timeout = 0;
 		return FALSE;
 	}
 
+	data = g_slice_new0(MsgQueueTimeoutData);
+	data->conn = conn;
+	data->sent_msgs = g_slist_prepend(data->sent_msgs, output_msg);
+
 	g_strlcpy(msg, output_msg->message, IRC_MSG_MAXLEN + 3);
 
 	for (i = 1; i < MSG_QUEUE_UNLOAD_AT_A_TIME; i++) {
-		output_msg = g_queue_peek_nth(priv->msg_queue, i);
+		output_msg = (IdleOutputPendingMsg *) g_queue_pop_head(priv->msg_queue);
+		data->sent_msgs = g_slist_prepend(data->sent_msgs, output_msg);
 
 		if ((output_msg != NULL) && ((strlen(msg) + strlen(output_msg->message)) < IRC_MSG_MAXLEN + 2))
 			strcat(msg, output_msg->message);
@@ -807,43 +901,93 @@ static gboolean msg_queue_timeout_cb(gpointer user_data) {
 			break;
 	}
 
-	if (idle_server_connection_iface_send(priv->conn, msg, &error)) {
-		for (j = 0; j < i; j++) {
-			output_msg = g_queue_pop_head(priv->msg_queue);
-			idle_output_pending_msg_free(output_msg);
-		}
-
-		priv->last_msg_sent = time(NULL);
-	} else {
-		IDLE_DEBUG("low-level network connection failed to send: %s", error->message);
-
-		g_error_free(error);
-	}
+	idle_server_connection_send_async(priv->conn, msg, NULL, _msg_queue_timeout_ready, data);
+	priv->msg_sending = TRUE;
 
 	return TRUE;
 }
 
+static void _add_msg_to_queue(IdleConnection *conn, gchar *msg, guint priority) {
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	IdleOutputPendingMsg *output_msg;
+
+	output_msg = idle_output_pending_msg_new();
+	output_msg->message = msg;
+	output_msg->priority = priority;
+
+	g_queue_insert_sorted(priv->msg_queue, output_msg, pending_msg_compare, NULL);
+	idle_connection_add_queue_timeout (conn);
+}
+
+static void _send_with_max_priority_ready(GObject *source_object, GAsyncResult *res, gpointer user_data) {
+	IdleServerConnection *sconn = IDLE_SERVER_CONNECTION(source_object);
+	SendWithPriorityData *data = (SendWithPriorityData *) user_data;
+	IdleConnection *conn = data->conn;
+	gchar *msg = data->msg;
+	guint priority = data->priority;
+	GError *error = NULL;
+
+	g_slice_free(SendWithPriorityData, data);
+
+	if (!idle_server_connection_send_finish(sconn, res, &error)) {
+		IDLE_DEBUG("idle_server_connection_send failed: %s", error->message);
+		g_error_free(error);
+		_add_msg_to_queue(conn, msg, priority);
+		return;
+	}
+
+	g_free(msg);
+}
+
+static void
+idle_connection_add_queue_timeout (IdleConnection *self)
+{
+  IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE (self);
+
+  if (priv->msg_queue_timeout == 0)
+    {
+      if (flush_queue_faster)
+        priv->msg_queue_timeout = g_timeout_add (MSG_QUEUE_TIMEOUT,
+            msg_queue_timeout_cb, self);
+      else
+        priv->msg_queue_timeout = g_timeout_add_seconds (MSG_QUEUE_TIMEOUT,
+            msg_queue_timeout_cb, self);
+    }
+}
+
+static void
+idle_connection_clear_queue_timeout (IdleConnection *self)
+{
+  IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE (self);
+
+  if (priv->msg_queue_timeout != 0)
+    {
+      g_source_remove (priv->msg_queue_timeout);
+      priv->msg_queue_timeout = 0;
+    }
+}
+
 /**
  * Queue a IRC command for sending, clipping it to IRC_MSG_MAXLEN bytes and appending the required <CR><LF> to it
  */
 static void _send_with_priority(IdleConnection *conn, const gchar *msg, guint priority) {
 	gchar cmd[IRC_MSG_MAXLEN + 3];
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	SendWithPriorityData *data;
 	int len;
-	GError *error = NULL;
 	gchar *converted;
 	GError *convert_error = NULL;
 	time_t curr_time = time(NULL);
-	IdleOutputPendingMsg *output_msg;
 
 	g_assert(msg != NULL);
 
 	/* Clip the message */
-
 	g_strlcpy(cmd, msg, IRC_MSG_MAXLEN + 1);
 
-	/* Append <CR><LF> */
+	/* Strip out any <CR>/<LF> which have crept in */
+	g_strdelimit (cmd, "\r\n", ' ');
 
+	/* Append <CR><LF> */
 	len = strlen(cmd);
 	cmd[len++] = '\r';
 	cmd[len++] = '\n';
@@ -856,26 +1000,22 @@ static void _send_with_priority(IdleConnection *conn, const gchar *msg, guint pr
 		converted = g_strdup(cmd);
 	}
 
-	if ((priority == SERVER_CMD_MAX_PRIORITY) || ((conn->parent.status == TP_CONNECTION_STATUS_CONNECTED)	&& (priv->msg_queue_timeout == 0)	&& (curr_time - priv->last_msg_sent > MSG_QUEUE_TIMEOUT))) {
+	if (priority == SERVER_CMD_MAX_PRIORITY ||
+	    (conn->parent.status == TP_CONNECTION_STATUS_CONNECTED &&
+	     priv->msg_queue_timeout == 0 &&
+	     curr_time - priv->last_msg_sent > MSG_QUEUE_TIMEOUT)) {
 		priv->last_msg_sent = curr_time;
 
-		if (!idle_server_connection_iface_send(priv->conn, converted, &error)) {
-			IDLE_DEBUG("server connection failed to send: %s", error->message);
-			g_error_free(error);
-		} else {
-			g_free(converted);
-			return;
-		}
+		data = g_slice_new0(SendWithPriorityData);
+		data->conn = conn;
+		data->msg = converted;
+		data->priority = priority;
+
+		idle_server_connection_send_async(priv->conn, converted, NULL, _send_with_max_priority_ready, data);
+		return;
 	}
 
-	output_msg = idle_output_pending_msg_new();
-	output_msg->message = converted;
-	output_msg->priority = priority;
-
-	g_queue_insert_sorted(priv->msg_queue, output_msg, pending_msg_compare, NULL);
-
-	if (!priv->msg_queue_timeout)
-		priv->msg_queue_timeout = g_timeout_add(MSG_QUEUE_TIMEOUT * 1000, msg_queue_timeout_cb, conn);
+	_add_msg_to_queue(conn, converted, priority);
 }
 
 void idle_connection_send(IdleConnection *conn, const gchar *msg) {
@@ -904,23 +1044,23 @@ idle_connection_get_max_message_length(IdleConnection *conn)
 
 static IdleParserHandlerResult _error_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
-	GHashTable *details = NULL;
 	TpConnectionStatus status = conn->parent.status;
 	TpConnectionStatusReason reason;
 	const gchar *msg;
 	const gchar *begin;
 	const gchar *end;
-	const gchar *error_name;
+	gchar *server_msg = NULL;
+	gint error;
 
 	switch (status) {
 		case TP_CONNECTION_STATUS_CONNECTING:
 			reason = TP_CONNECTION_STATUS_REASON_AUTHENTICATION_FAILED;
-			error_name = TP_ERROR_STR_AUTHENTICATION_FAILED;
+			error = TP_ERROR_AUTHENTICATION_FAILED;
 			break;
 
 		case TP_CONNECTION_STATUS_CONNECTED:
 			reason = TP_CONNECTION_STATUS_REASON_NETWORK_ERROR;
-			error_name = TP_ERROR_STR_NETWORK_ERROR;
+			error = TP_ERROR_NETWORK_ERROR;
 			break;
 
 		default:
@@ -932,21 +1072,16 @@ static IdleParserHandlerResult _error_handler(IdleParser *parser, IdleParserMess
 	end = strrchr(msg, ')');
 
 	if (begin != NULL && end != NULL && begin < end - 1) {
-		gchar *server_msg;
 		guint length;
 
 		begin++;
 		end--;
 		length = end - begin + 1;
 		server_msg = g_strndup(begin, length);
-		details = tp_asv_new ("server-message", G_TYPE_STRING, server_msg, NULL);
-		g_free(server_msg);
 	}
 
-	tp_base_connection_disconnect_with_dbus_error(TP_BASE_CONNECTION(conn), error_name, details, reason);
-
-	if (details != NULL)
-		g_hash_table_unref(details);
+	_connection_disconnect_with_error(conn, reason, "server-message", TP_ERRORS, error, (server_msg == NULL) ? "" : server_msg);
+	g_free(server_msg);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
@@ -1095,17 +1230,13 @@ static void connection_connect_cb(IdleConnection *conn, gboolean success, TpConn
 
 static void connection_disconnect_cb(IdleConnection *conn, TpConnectionStatusReason reason) {
 	TpBaseConnection *base = TP_BASE_CONNECTION(conn);
-	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
 
 	if (base->status == TP_CONNECTION_STATUS_DISCONNECTED)
 		g_idle_add(_finish_shutdown_idle_func, base);
 	else
 		tp_base_connection_change_status(base, TP_CONNECTION_STATUS_DISCONNECTED, reason);
 
-	if (priv->msg_queue_timeout) {
-		g_source_remove(priv->msg_queue_timeout);
-		priv->msg_queue_timeout = 0;
-	}
+	idle_connection_clear_queue_timeout (conn);
 }
 
 void _queue_alias_changed(IdleConnection *conn, TpHandle handle, const gchar *alias) {
@@ -1121,17 +1252,11 @@ void _queue_alias_changed(IdleConnection *conn, TpHandle handle, const gchar *al
 	if (!priv->queued_aliases)
 		priv->queued_aliases = g_ptr_array_new();
 
-	GValue value = {0, };
-
-	g_value_init(&value, IDLE_TP_ALIAS_PAIR_TYPE);
-	g_value_take_boxed(&value, dbus_g_type_specialized_construct(IDLE_TP_ALIAS_PAIR_TYPE));
-
-	dbus_g_type_struct_set(&value,
-			0, handle,
-			1, alias,
-			G_MAXUINT);
-
-	g_ptr_array_add(priv->queued_aliases, g_value_get_boxed(&value));
+	g_ptr_array_add(priv->queued_aliases,
+		tp_value_array_build (2,
+			G_TYPE_UINT, handle,
+			G_TYPE_STRING, alias,
+			G_TYPE_INVALID));
 }
 
 static GQuark _canon_nick_quark() {
@@ -1178,6 +1303,75 @@ static void idle_connection_get_alias_flags(TpSvcConnectionInterfaceAliasing *if
 	tp_svc_connection_interface_aliasing_return_from_get_alias_flags(context, 0);
 }
 
+static const gchar *
+gimme_an_alias (
+    TpHandleRepoIface *repo,
+    TpHandle handle)
+{
+  const gchar *alias = tp_handle_get_qdata (repo, handle, _canon_nick_quark());
+
+  if (alias != NULL)
+    return alias;
+  else
+    return tp_handle_inspect (repo, handle);
+}
+
+static void
+conn_aliasing_fill_contact_attributes (
+    GObject *obj,
+    const GArray *contacts,
+    GHashTable *attributes_hash)
+{
+  IdleConnection *self = IDLE_CONNECTION (obj);
+  TpHandleRepoIface *repo = tp_base_connection_get_handles (
+      TP_BASE_CONNECTION (self), TP_HANDLE_TYPE_CONTACT);
+  guint i;
+
+  for (i = 0; i < contacts->len; i++)
+    {
+      TpHandle handle = g_array_index (contacts, TpHandle, i);
+      const gchar *alias = gimme_an_alias (repo, handle);
+
+      g_assert (alias != NULL);
+
+      tp_contacts_mixin_set_contact_attribute (attributes_hash,
+          handle, TP_IFACE_CONNECTION_INTERFACE_ALIASING"/alias",
+          tp_g_value_slice_new_string (alias));
+    }
+}
+
+static void
+idle_connection_get_aliases (
+    TpSvcConnectionInterfaceAliasing *iface,
+    const GArray *handles,
+    DBusGMethodInvocation *context)
+{
+  TpHandleRepoIface *repo = tp_base_connection_get_handles (
+      TP_BASE_CONNECTION (iface), TP_HANDLE_TYPE_CONTACT);
+  GError *error = NULL;
+
+  if (!tp_handles_are_valid (repo, handles, FALSE, &error))
+    {
+      dbus_g_method_return_error(context, error);
+      g_error_free(error);
+      return;
+    }
+
+  GHashTable *aliases = g_hash_table_new (NULL, NULL);
+
+  for (guint i = 0; i < handles->len; i++)
+    {
+      TpHandle handle = g_array_index (handles, TpHandle, i);
+
+      g_hash_table_insert (aliases, GUINT_TO_POINTER (handle),
+          (gpointer) gimme_an_alias (repo, handle));
+    }
+
+  tp_svc_connection_interface_aliasing_return_from_get_aliases (context,
+      aliases);
+  g_hash_table_unref (aliases);
+}
+
 static void idle_connection_request_aliases(TpSvcConnectionInterfaceAliasing *iface, const GArray *handles, DBusGMethodInvocation *context) {
 	TpHandleRepoIface *repo = tp_base_connection_get_handles(TP_BASE_CONNECTION(iface), TP_HANDLE_TYPE_CONTACT);
 	GError *error = NULL;
@@ -1192,15 +1386,10 @@ static void idle_connection_request_aliases(TpSvcConnectionInterfaceAliasing *if
 	for (guint i = 0; i < handles->len; i++) {
 		TpHandle handle = g_array_index(handles, TpHandle, i);
 
-		const gchar *alias = tp_handle_get_qdata(repo, handle, _canon_nick_quark());
-		if (!alias)
-			alias = tp_handle_inspect(repo, handle);
-
-		aliases[i] = alias;
+		aliases[i] = gimme_an_alias (repo, handle);
 	}
 
 	tp_svc_connection_interface_aliasing_return_from_request_aliases(context, aliases);
-
 	g_free(aliases);
 }
 
@@ -1309,6 +1498,7 @@ static void _aliasing_iface_init(gpointer g_iface, gpointer iface_data) {
 #define IMPLEMENT(x) tp_svc_connection_interface_aliasing_implement_##x (\
 		klass, idle_connection_##x)
 	IMPLEMENT(get_alias_flags);
+	IMPLEMENT(get_aliases);
 	IMPLEMENT(request_aliases);
 	IMPLEMENT(set_aliases);
 #undef IMPLEMENT
