@@ -22,6 +22,7 @@
 #define  _GNU_SOURCE
 
 #include "idle-parser.h"
+#include "config.h"
 
 #include "idle-connection.h"
 #include "idle-muc-channel.h"
@@ -72,6 +73,7 @@ struct _MessageSpec {
  * '.' - Same as ':', but optional
  */
 static const MessageSpec message_specs[] = {
+	{"ERROR", "I:", IDLE_PARSER_CMD_ERROR},
 	{"PING", "Is", IDLE_PARSER_CMD_PING},
 
 	{"INVITE", "cIcr", IDLE_PARSER_PREFIXCMD_INVITE},
@@ -101,11 +103,20 @@ static const MessageSpec message_specs[] = {
 	{"366", "IIIr", IDLE_PARSER_NUMERIC_NAMEREPLY_END},
 	{"433", "III", IDLE_PARSER_NUMERIC_NICKNAMEINUSE},
 	{"401", "IIIc", IDLE_PARSER_NUMERIC_NOSUCHNICK},
+	{"402", "IIIs:", IDLE_PARSER_NUMERIC_NOSUCHSERVER},
 	{"306", "III", IDLE_PARSER_NUMERIC_NOWAWAY},
 	{"332", "IIIr:", IDLE_PARSER_NUMERIC_TOPIC},
 	{"333", "IIIrcd", IDLE_PARSER_NUMERIC_TOPIC_STAMP},
+	{"263", "IIIs:", IDLE_PARSER_NUMERIC_TRYAGAIN},
 	{"305", "III", IDLE_PARSER_NUMERIC_UNAWAY},
 	{"001", "IIc", IDLE_PARSER_NUMERIC_WELCOME},
+	{"319", "IIIc.", IDLE_PARSER_NUMERIC_WHOISCHANNELS},
+	{"378", "IIIc:", IDLE_PARSER_NUMERIC_WHOISHOST},
+	{"330", "IIIcs:", IDLE_PARSER_NUMERIC_WHOISLOGGEDIN},
+	{"313", "IIIc:", IDLE_PARSER_NUMERIC_WHOISOPERATOR},
+	{"307", "IIIc:", IDLE_PARSER_NUMERIC_WHOISREGNICK},
+	{"671", "IIIc:", IDLE_PARSER_NUMERIC_WHOISSECURE},
+	{"312", "IIIcs:", IDLE_PARSER_NUMERIC_WHOISSERVER},
 	{"311", "IIIcssI:", IDLE_PARSER_NUMERIC_WHOISUSER},
 	{"317", "IIIcd", IDLE_PARSER_NUMERIC_WHOISIDLE},
 
@@ -197,6 +208,20 @@ static void _parse_message(IdleParser *parser, const gchar *split_msg);
 static void _parse_and_forward_one(IdleParser *parser, gchar **tokens, IdleParserMessageCode code, const gchar *format);
 static gboolean _parse_atom(IdleParser *parser, GValueArray *arr, char atom, const gchar *token, TpHandleSet *contact_reffed, TpHandleSet *room_reffed);
 
+#ifndef HAVE_STRNLEN
+static size_t
+strnlen(const char *msg, size_t maxlen)
+{
+	size_t i;
+
+	for (i=0; i<maxlen; i++)
+		if (msg[i] == '\0')
+			break;
+
+	return i;
+}
+#endif
+
 void idle_parser_receive(IdleParser *parser, const gchar *msg) {
 	IdleParserPrivate *priv = IDLE_PARSER_GET_PRIVATE(parser);
 	guint i;
@@ -461,10 +486,12 @@ static gboolean _parse_atom(IdleParser *parser, GValueArray *arr, char atom, con
 			}
 
 			id = g_strdup(token);
-			bang = strchr(id, '!');
 
-			if (bang)
-				*bang = '\0';
+			if (atom != 'r') {
+				bang = strchr(id, '!');
+				if (bang)
+					*bang = '\0';
+			}
 
 			if (atom == 'r') {
 				if ((handle = tp_handle_ensure(room_repo, id, NULL, NULL))) {
