@@ -46,7 +46,8 @@ G_BEGIN_DECLS
 	(G_TYPE_INSTANCE_GET_CLASS((obj), IDLE_TYPE_PARSER, IdleParserClass))
 
 typedef enum {
-	IDLE_PARSER_CMD_PING = 0,
+	IDLE_PARSER_CMD_ERROR = 0,
+	IDLE_PARSER_CMD_PING,
 
 	IDLE_PARSER_PREFIXCMD_INVITE,
 	IDLE_PARSER_PREFIXCMD_JOIN,
@@ -75,11 +76,20 @@ typedef enum {
 	IDLE_PARSER_NUMERIC_NAMEREPLY_END,
 	IDLE_PARSER_NUMERIC_NICKNAMEINUSE,
 	IDLE_PARSER_NUMERIC_NOSUCHNICK,
+	IDLE_PARSER_NUMERIC_NOSUCHSERVER,
 	IDLE_PARSER_NUMERIC_NOWAWAY,
 	IDLE_PARSER_NUMERIC_TOPIC,
 	IDLE_PARSER_NUMERIC_TOPIC_STAMP,
+	IDLE_PARSER_NUMERIC_TRYAGAIN,
 	IDLE_PARSER_NUMERIC_UNAWAY,
 	IDLE_PARSER_NUMERIC_WELCOME,
+	IDLE_PARSER_NUMERIC_WHOISCHANNELS,
+	IDLE_PARSER_NUMERIC_WHOISHOST,
+	IDLE_PARSER_NUMERIC_WHOISLOGGEDIN,
+	IDLE_PARSER_NUMERIC_WHOISOPERATOR,
+	IDLE_PARSER_NUMERIC_WHOISREGNICK,
+	IDLE_PARSER_NUMERIC_WHOISSECURE,
+	IDLE_PARSER_NUMERIC_WHOISSERVER,
 	IDLE_PARSER_NUMERIC_WHOISUSER,
 	IDLE_PARSER_NUMERIC_WHOISIDLE,
 
