@@ -12,6 +12,8 @@ from twisted.words.protocols import irc
 from twisted.internet import reactor, ssl
 
 def make_irc_event(type, data):
+    if data:
+        data[-1] = data[-1].rstrip('\r\n')
     event = servicetest.Event(type, data=data)
     return event
 
@@ -125,24 +127,23 @@ class BaseIRCServer(irc.IRC):
         self.sendMessage('366', self.nick, room, ':End of /NAMES list', prefix='idle.test.server')
 
     def handleQUIT(self, args, prefix):
-        quit_msg = ' '.join(args).rstrip('\r\n')
+        quit_msg = ' '.join(args)
         self.sendMessage('ERROR', ':Closing Link: idle.test.server (Quit: %s)' % quit_msg)
         self.transport.loseConnection()
 
     def sendWelcome(self):
         self.sendMessage('001', self.nick, ':Welcome to the test IRC Network', prefix='idle.test.server')
 
-    def dataReceived(self, data):
-        (_prefix, _command, _args) = irc.parsemsg(data)
-        self.event_func(make_irc_event('stream-%s' % _command, _args))
+    def handleCommand(self, command, prefix, params):
+        self.event_func(make_irc_event('stream-%s' % command, params))
         try:
-            f = getattr(self, 'handle%s' % _command)
+            f = getattr(self, 'handle%s' % command)
             try:
-                f(_args, _prefix)
+                f(params, prefix)
             except Exception, e:
                 self.log('handler failed: %s' % e)
         except Exception, e:
-            self.log('No handler for command %s: %s' % (_command, e))
+            self.log('No handler for command %s: %s' % (command, e))
 
 class SSLIRCServer(BaseIRCServer):
     def __init__(self, event_func):
@@ -156,6 +157,10 @@ class SSLIRCServer(BaseIRCServer):
         return reactor.listenSSL(port, factory,
                 ssl.DefaultOpenSSLContextFactory(key_file, cert_file))
 
+def sync_stream(q, stream):
+    stream.sendMessage('PING', 'sup')
+    q.expect('stream-PONG')
+
 def install_colourer():
     def red(s):
         return '\x1b[31m%s\x1b[0m' % s
